<?php

namespace BitApps\BTCBI_PRO\Triggers\Paymattic;

use BitCode\FI\Flow\Flow;
use BitCode\FI\Core\Util\Helper;
use BitApps\BTCBI_PRO\Triggers\TriggerController;

final class PaymatticController
{
    public static function info()
    {
        return [
            'name'              => 'Paymattic',
            'title'             => __('Paymattic is a WordPress Payment form plugin.', 'bit-integrations-pro'),
            'type'              => 'custom_form_submission',
            'is_active'         => static::isPluginInstalled(),
            'documentation_url' => '#',
            'tutorial_url'      => '#',
            'tasks'             => [
                'action' => 'paymattic/get',
                'method' => 'get',
            ],
            'fetch' => [
                'action' => 'paymattic/test',
                'method' => 'post',
            ],
            'fetch_remove' => [
                'action' => 'paymattic/test/remove',
                'method' => 'post',
            ],
            'isPro' => true
        ];
    }

    public function getAllTasks()
    {
        if (!static::isPluginInstalled()) {
            wp_send_json_error(\sprintf(__('%s is not installed or activated', 'bit-integrations-pro'), 'Paymattic'));
        }

        wp_send_json_success([
            ['form_name' => __('Form Submission', 'bit-integrations-pro'), 'triggered_entity_id' => 'paymattic_after_form_submission_complete', 'skipPrimaryKey' => false],
        ]);
    }

    public function getTestData($data)
    {
        return TriggerController::getTestData($data->triggered_entity_id);
    }

    public function removeTestData($data)
    {
        return TriggerController::removeTestData($data, $data->triggered_entity_id);
    }

    public static function handlePaymentFormCompleted($submission, $form_id)
    {
        if (! isset($form_id)) {
            return;
        }

        $formData = Helper::prepareFetchFormatFields((array) json_decode(wp_json_encode($submission)));
        if (empty($formData) || !\is_array($formData)) {
            return;
        }

        Helper::setTestData('btcbi_paymattic_after_form_submission_complete_test', array_values($formData), 'form_id.value', $formData['form_id']['value']);

        $flows = Flow::exists('Paymattic', 'paymattic_after_form_submission_complete');
        if (!$flows) {
            return;
        }

        foreach ($flows as $flow) {
            $flowDetails = Helper::parseFlowDetails($flow->flow_details);

            if (!isset($flowDetails->primaryKey) || !Helper::isPrimaryKeysMatch($formData, $flowDetails->primaryKey)) {
                continue;
            }

            Flow::execute('Paymattic', 'paymattic_after_form_submission_complete', array_column($formData, 'value', 'name'), [$flow]);
        }

        return ['type' => 'success'];
    }

    private static function isPluginInstalled()
    {
        return \defined('WPPAYFORM_VERSION');
    }
}
